package javax.management.relation;

import java.security.AccessController;
import com.sun.jmx.mbeanserver.GetPropertyAction;
import java.io.ObjectOutputStream;
import java.io.IOException;
import com.sun.jmx.mbeanserver.Util;
import java.io.ObjectInputStream;
import java.util.HashSet;
import java.util.ArrayList;
import java.util.List;
import com.sun.jmx.defaults.JmxProperties;
import java.util.HashMap;
import java.util.Map;
import java.io.ObjectStreamField;

public class RelationTypeSupport implements RelationType
{
    private static final long oldSerialVersionUID = -8179019472410837190L;
    private static final long newSerialVersionUID = 4611072955724144607L;
    private static final ObjectStreamField[] oldSerialPersistentFields;
    private static final ObjectStreamField[] newSerialPersistentFields;
    private static final long serialVersionUID;
    private static final ObjectStreamField[] serialPersistentFields;
    private static boolean compat;
    private String typeName;
    private Map<String, RoleInfo> roleName2InfoMap;
    private boolean isInRelationService;
    
    public RelationTypeSupport(String param1, RoleInfo[] array) throws IllegalArgumentException, InvalidRelationTypeException {
        this.typeName = null;
        this.roleName2InfoMap = new HashMap();
        this.isInRelationService = false;
        if (param1 == null || array == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationTypeSupport.class.getName(), "RelationTypeSupport", param1);
        this.initMembers(param1, array);
        JmxProperties.RELATION_LOGGER.exiting(RelationTypeSupport.class.getName(), "RelationTypeSupport");
        return;
    }
    
    protected RelationTypeSupport(String s) {
        this.typeName = null;
        this.roleName2InfoMap = new HashMap();
        this.isInRelationService = false;
        if (s == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationTypeSupport.class.getName(), "RelationTypeSupport", s);
        this.typeName = s;
        JmxProperties.RELATION_LOGGER.exiting(RelationTypeSupport.class.getName(), "RelationTypeSupport");
        return;
    }
    
    @Override
    public String getRelationTypeName() {
        return this.typeName;
    }
    
    @Override
    public List<RoleInfo> getRoleInfos() {
        return new ArrayList(this.roleName2InfoMap.values());
    }
    
    @Override
    public RoleInfo getRoleInfo(String s) throws IllegalArgumentException, RoleInfoNotFoundException {
        if (s == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationTypeSupport.class.getName(), "getRoleInfo", s);
        RoleInfo roleInfo = (RoleInfo)this.roleName2InfoMap.get(s);
        if (roleInfo == null) {
            StringBuilder sb = new StringBuilder();
            sb.append("No role info for role ");
            sb.append(s);
            throw new RoleInfoNotFoundException(sb.toString());
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationTypeSupport.class.getName(), "getRoleInfo");
        return roleInfo;
    }
    
    protected void addRoleInfo(RoleInfo param1) throws IllegalArgumentException, InvalidRelationTypeException {
        if (param1 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationTypeSupport.class.getName(), "addRoleInfo", param1);
        if (this.isInRelationService) {
            throw new RuntimeException("Relation type cannot be updated as it is declared in the Relation Service.");
        }
        String name = param1.getName();
        if (this.roleName2InfoMap.containsKey(name)) {
            StringBuilder sb = new StringBuilder();
            sb.append("Two role infos provided for role ");
            sb.append(name);
            throw new InvalidRelationTypeException(sb.toString());
        }
        this.roleName2InfoMap.put(name, new RoleInfo(param1));
        JmxProperties.RELATION_LOGGER.exiting(RelationTypeSupport.class.getName(), "addRoleInfo");
        return;
    }
    
    void setRelationServiceFlag(boolean isInRelationService) {
        this.isInRelationService = isInRelationService;
    }
    
    private void initMembers(String s, RoleInfo[] array) throws IllegalArgumentException, InvalidRelationTypeException {
        if (s == null || array == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationTypeSupport.class.getName(), "initMembers", s);
        this.typeName = s;
        checkRoleInfos(array);
        for (int i = 0; i < array.length; ++i) {
            RoleInfo roleInfo = array[i];
            this.roleName2InfoMap.put(roleInfo.getName(), new RoleInfo(roleInfo));
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationTypeSupport.class.getName(), "initMembers");
        return;
    }
    
    static void checkRoleInfos(RoleInfo[] array) throws IllegalArgumentException, InvalidRelationTypeException {
        if (array == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        if (array.length == 0) {
            throw new InvalidRelationTypeException("No role info provided.");
        }
        HashSet<String> set = new HashSet();
        int i = 0;
        while (i < array.length) {
            RoleInfo roleInfo = array[i];
            if (roleInfo == null) {
                throw new InvalidRelationTypeException("Null role info provided.");
            }
            String name = roleInfo.getName();
            if (set.contains(name)) {
                StringBuilder sb = new StringBuilder();
                sb.append("Two role infos provided for role ");
                sb.append(name);
                throw new InvalidRelationTypeException(sb.toString());
            }
            set.add(name);
            ++i;
        }
        return;
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        if (RelationTypeSupport.compat) {
            ObjectInputStream.GetField fields = objectInputStream.readFields();
            this.typeName = (String)fields.get("myTypeName", null);
            if (fields.defaulted("myTypeName")) {
                throw new NullPointerException("myTypeName");
            }
            this.roleName2InfoMap = (Map)Util.cast(fields.get("myRoleName2InfoMap", null));
            if (fields.defaulted("myRoleName2InfoMap")) {
                throw new NullPointerException("myRoleName2InfoMap");
            }
            this.isInRelationService = fields.get("myIsInRelServFlg", false);
            if (fields.defaulted("myIsInRelServFlg")) {
                throw new NullPointerException("myIsInRelServFlg");
            }
        }
        else {
            objectInputStream.defaultReadObject();
        }
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        if (RelationTypeSupport.compat) {
            ObjectOutputStream.PutField putFields = objectOutputStream.putFields();
            putFields.put("myTypeName", this.typeName);
            putFields.put("myRoleName2InfoMap", this.roleName2InfoMap);
            putFields.put("myIsInRelServFlg", this.isInRelationService);
            objectOutputStream.writeFields();
        }
        else {
            objectOutputStream.defaultWriteObject();
        }
    }
    
    static {
        oldSerialPersistentFields = new ObjectStreamField[] { new ObjectStreamField("myTypeName", String.class), new ObjectStreamField("myRoleName2InfoMap", HashMap.class), new ObjectStreamField("myIsInRelServFlg", Boolean.TYPE) };
        newSerialPersistentFields = new ObjectStreamField[] { new ObjectStreamField("typeName", String.class), new ObjectStreamField("roleName2InfoMap", Map.class), new ObjectStreamField("isInRelationService", Boolean.TYPE) };
        RelationTypeSupport.compat = false;
        try {
            String s = (String)AccessController.doPrivileged(new GetPropertyAction("jmx.serial.form"));
            RelationTypeSupport.compat = (s != null && s.equals("1.0"));
        }
        catch (Exception ex) {}
        if (RelationTypeSupport.compat) {
            serialPersistentFields = RelationTypeSupport.oldSerialPersistentFields;
            serialVersionUID = -8179019472410837190L;
        }
        else {
            serialPersistentFields = RelationTypeSupport.newSerialPersistentFields;
            serialVersionUID = 4611072955724144607L;
        }
    }
}
